<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet
 version="1.0"
 xmlns="http://www.w3.org/2000/svg"
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns:xalan="http://xml.apache.org/xalan"
 xmlns:xlink="http://www.w3.org/1999/xlink">

<xsl:include href="xradar://stylesheets/radar/statics/graph/common-graph.xsl"/>

<xsl:template name="scatter-graph">
	<xsl:param name="main-sequence"/>
	<xsl:param name="text-x-axis"/>
	<xsl:param name="text-y-axis"/>
	<xsl:param name="point-nodeset"/>
	<xsl:param name="graph-additions"/>
	<xsl:param name="include-quadrants"/>
	<xsl:param name="include-grid-lines"/>
	<xsl:param name="include-pareto-lines"/>
	<xsl:param name="include-filtered-line"/>

   <svg width="13cm" height="13cm" viewBox="0 0 1300 1300"
     xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink">


   	<!-- Include common libraries -->
   	<xsl:call-template name="common-graph">
	  	<xsl:with-param name="text-x-axis" select="$text-x-axis"/>
	  	<xsl:with-param name="text-y-axis" select="$text-y-axis"/>
	</xsl:call-template>

   	<text
   		xml:space="preserve"
   		style="font-size:60px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#008000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Bitstream Vera Sans"
   		x="570"
   		y="100"
   		id="main-sequence-label"
   		transform="matrix(0.7087326,0.7054772,-0.7054772,0.7087326,0,0)">
   		<tspan
   			x="600"
   			y="60"><xsl:value-of select="$main-sequence"/>
   		</tspan>
   	</text>
   	<text
   		xml:space="preserve"
   		style="font-size:50px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#008000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Bitstream Vera Sans"
   		x="1049"
   		y="1246"
   		id="scale-x-max"><tspan
   			x="1049"
   			y="1246">1.0</tspan></text>
   	<text
   		xml:space="preserve"
   		style="font-size:50px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#008000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Bitstream Vera Sans"
   		x="0"
   		y="1225"
   		id="scale-zero"><tspan
   			x="20"
   			y="1225">0</tspan></text>
   	<text
   		xml:space="preserve"
   		style="font-size:50px;font-style:normal;font-variant:normal;font-weight:normal;font-stretch:normal;text-align:start;line-height:125%;writing-mode:lr-tb;text-anchor:start;fill:#008000;fill-opacity:1;stroke:none;stroke-width:1px;stroke-linecap:butt;stroke-linejoin:miter;stroke-opacity:1;font-family:Bitstream Vera Sans"
   		x="0"
   		y="190"
   		id="scale-y-max"><tspan
   			x="10"
   			y="180">1.0</tspan></text>

	<xsl:call-template name="graph-markers"/>

	  <xsl:if test="$include-quadrants = 'true'">
	  	<xsl:call-template name="quadrants-on-graph"/>
	  </xsl:if>

	  <xsl:if test="$include-grid-lines= 'true'">
	  	<xsl:call-template name="grid-lines"/>
	  </xsl:if>

	  <xsl:if test="$include-pareto-lines= 'true'">
	  	<xsl:call-template name="pareto-lines"/>
	  </xsl:if>

	  <xsl:copy-of select="xalan:nodeset($graph-additions)"/>

	  <xsl:variable name="pointset" select="xalan:nodeset($point-nodeset)"/>
	  <xsl:apply-templates select="$pointset/points/point" mode="red"/>

	  <xsl:if test="$include-filtered-line= 'true'">
	  	<xsl:call-template name="filtered-graph">
			<xsl:with-param name="pointset" select="$pointset"/>
		</xsl:call-template>
	  </xsl:if>

	</svg>

</xsl:template>

<!-- plots a point on the graph -->
<xsl:template match="point" mode="red">
	<circle r="5"
	    	fill="red"
	    	stroke="red"
	    	stroke-width="1"  >
		<xsl:attribute name="cx">
			<xsl:value-of select="100 + 1000 * x-coordinate"/>
		</xsl:attribute>
		<xsl:attribute name="cy">
			<xsl:value-of select="1200 - 1000 * y-coordinate"/>
		</xsl:attribute>
	</circle>
</xsl:template>

<xsl:template match="point" mode="black">
	<circle r="3"
	    	fill="red"
	    	stroke="black"
	    	stroke-width="1"  >
		<xsl:attribute name="cx">
			<xsl:value-of select="100 + 1000 * x-coordinate"/>
		</xsl:attribute>
		<xsl:attribute name="cy">
			<xsl:value-of select="1200 - 1000 * y-coordinate"/>
		</xsl:attribute>
	</circle>
</xsl:template>

<!-- plots a lowpassed filtered graph-->
<xsl:template name="filtered-graph">
	<xsl:param name="pointset"/>

	<xsl:variable name="rtf-filtered-points">
		<points>
			<xsl:call-template name="filtered-point">
				<xsl:with-param name="points" select="$pointset/points"/>
				<xsl:with-param name="point_number" select="'1'"/>
				<xsl:with-param name="y_value_1" select="'0'"/>
				<xsl:with-param name="y_value_2" select="'0'"/>
				<xsl:with-param name="y_filtered_value_1" select="'0'"/>
				<xsl:with-param name="y_filtered_value_2" select="'h0'"/>
			</xsl:call-template>
		</points>
	</xsl:variable>

	<xsl:apply-templates select="xalan:nodeset($rtf-filtered-points)/points/point" mode="black"/>
</xsl:template>


<!-- finds filtered points recursively-->
<xsl:template name="filtered-point">
	<xsl:param name="points"/>
	<xsl:param name="point_number"/>
	<xsl:param name="y_value_1"/>
	<xsl:param name="y_value_2"/>
	<xsl:param name="y_filtered_value_1"/>
	<xsl:param name="y_filtered_value_2"/>


	<xsl:variable name="a_0" select="'0.1'"/>
	<xsl:variable name="a_1" select="'0.14'"/>
	<xsl:variable name="a_2" select="$a_0"/>
	<xsl:variable name="b_1" select="'-1.14'"/>
	<xsl:variable name="b_2" select="'0.43'"/>

	<xsl:variable name="x_value_0" select="$points/point[position()=$point_number]/x-coordinate"/>
	<xsl:variable name="y_value_0" select="$points/point[position()=$point_number]/y-coordinate"/>

	<xsl:variable name="y_filtered_value_0" select="$a_0 * $y_value_0 + $a_1 * $y_value_1 + $a_2 * $y_value_2  - $b_1 * $y_filtered_value_1 - $b_2 * $y_filtered_value_1"/>


	<point>
		<x-coordinate>
			<xsl:value-of select="x_value_0"/>
		</x-coordinate>
		<y-coordinate>
			<xsl:value-of select="y_filtered_value_0"/>
		</y-coordinate>
	</point>

	<xsl:if test="$point_number &lt; count($points/point)">
		<xsl:call-template name="filtered-point">
			<xsl:with-param name="points" select="$points"/>
			<xsl:with-param name="point_number" select="$point_number + 1"/>
			<xsl:with-param name="y_value_1" select="y_value_0"/>
			<xsl:with-param name="y_value_2" select="y_value_1"/>
			<xsl:with-param name="y_filtered_value_1" select="y_filtered_value_0"/>
			<xsl:with-param name="y_filtered_value_2" select="y_filtered_value_1"/>
		</xsl:call-template>
	</xsl:if>
</xsl:template>

</xsl:stylesheet>
