<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects. 
======================================================================= -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  
   
<xsl:template match="Packages" mode="unused-code-analysis">	
   <html>
      <head>
          <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
      </head>
      <body>
        <xsl:call-template name="page-header">
  	  		<xsl:with-param name="main-menu-page-name" select="'analysis'"/>
  	  		<xsl:with-param name="page-name" select="'redundant code analysis'"/>
  	  		<xsl:with-param name="root-element" select=".."/>
  	    </xsl:call-template>         
  		
  		<h2>Totals</h2>
  		
			
		<xsl:call-template name="unused-code-in-subsystem">
			<xsl:with-param name="packages-element" select="current()"/>
		</xsl:call-template>
      	
      	  		
  		<h2>Details</h2>
  		<p>
  		Feel free to copy this table to excel or some other spreadsheet for further processing.
  		</p>
  		<table width="100%" class="details">
	        <tr>
	          <th>Subsystem</th>
	          <th>Class</th>
	          <th>Can be removed</th>
	          <th>Unused private fields</th>
	          <th>Unused private methods</th>
	          <th>Unused local variables</th>	          	          
	          <th>Unused formal parameters</th>
	          <th>Unused imports</th>
	          
	          <th>Empty catch blocks</th>
	          <th>Empty if stmts</th>
	          <th>Empty while stmts</th>
	          <th>Empty try blocks</th>
	          <th>Empty finally blocks</th>
	          <th>Empty switch statements</th>
	          
	        </tr>	        
	        <xsl:for-each select="Package">
		  		<xsl:apply-templates select="AbstractClasses/Class" mode="unused-code-per-class"/>
		  		<xsl:apply-templates select="ConcreteClasses/Class" mode="unused-code-per-class"/>
		  	</xsl:for-each>	
      	</table>	
      </body>
   </html>
</xsl:template>

<xsl:template name="unused-code-in-subsystem">
	<xsl:param name="packages-element"/>
	<xsl:variable name="total-classes" select="sum(Package/Stats/TotalClasses)"/>
	<xsl:variable name="removable-classes" select="$total-classes - sum(Package/Stats/business-critical-classes)"/>	
	
	<table width="100%" class="details">
        <tr>	        
          <th>Area</th>
          <th>Removable classes (% of total)</th>
          <th>Unused private fields</th>
          <th>Unused private methods</th>
          <th>Unused local variables</th>	          	          
          <th>Unused formal parameters</th>
          <th>Unused imports</th>
          
          <th>Empty catch blocks</th>
          <th>Empty if stmts</th>
          <th>Empty while stmts</th>
          <th>Empty try blocks</th>
          <th>Empty finally blocks</th>
          <th>Empty switch statements</th>
        </tr>	        
        
        <tr>
		  <td><b>All Classes</b></td>
		  <td>          	
		  		<b><xsl:value-of select="$removable-classes"/> 
		  		(<xsl:value-of select="format-number(100 * $removable-classes div $total-classes, '#.##')"/> %)</b>          			  	
		  </td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedPrivateField'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedPrivateMethod'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedLocalVariable'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedFormalParameter'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedImports'])"/></b></td>
		  
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyCatchBlock'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyIfStmt'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyWhileStmt'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyTryBlock'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyFinallyBlock'])"/></b></td>
		  <td><b><xsl:value-of select="count(Package/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptySwitchStatements'])"/></b></td>
		</tr>
		
		<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
			<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
			<xsl:variable name="total-classes-in-subsystem" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/TotalClasses)"/>
			<xsl:variable name="removable-classes-in-subsystem" select="$total-classes-in-subsystem - sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/business-critical-classes)"/>			
			<tr>
			  <td><xsl:value-of select="$subsystem-name"/></td>
			  <td>          	
			  		<xsl:value-of select="$removable-classes-in-subsystem"/> 
			  		(<xsl:value-of select="format-number(100 * $removable-classes-in-subsystem div $total-classes-in-subsystem, '#.##')"/> %)          			  	
			  </td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedPrivateField'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedPrivateMethod'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedLocalVariable'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedFormalParameter'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedImports'])"/></td>
			  
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyCatchBlock'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyIfStmt'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyWhileStmt'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyTryBlock'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyFinallyBlock'])"/></td>
			  <td><xsl:value-of select="count($packages-element/Package[subsystem/@name = $subsystem-name]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptySwitchStatements'])"/></td>
			</tr>										
	 	</xsl:for-each>
	 	
	 	<xsl:variable name="total-classes-not-classified" select="sum($packages-element/Package[not(subsystem)]/Stats/TotalClasses)"/>
		<xsl:variable name="removable-classes-not-classified" select="$total-classes-not-classified - sum($packages-element/Package[not(subsystem)]/Stats/business-critical-classes)"/>
	 	<tr>
		  <td>Not Classified</td>
		  <td>          	
		  		<xsl:value-of select="$removable-classes-not-classified"/> 
		  		(<xsl:value-of select="format-number(100 * $removable-classes-not-classified div $total-classes-not-classified, '#.##')"/> %)          			  	
		  </td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedPrivateField'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedPrivateMethod'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedLocalVariable'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedFormalParameter'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'UnusedImports'])"/></td>
		  
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyCatchBlock'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyIfStmt'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyWhileStmt'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyTryBlock'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptyFinallyBlock'])"/></td>
		  <td><xsl:value-of select="count($packages-element/Package[not(subsystem)]/child::node()[name() = 'AbstractClasses' or name() = 'ConcreteClasses']/Class/violations/violation[@rule = 'EmptySwitchStatements'])"/></td>
		</tr>
	 	
 	</table>
</xsl:template>

<xsl:template match="Class" mode="unused-code-per-class">
	<xsl:if test="(not(business-critical)) or (count(violations/violation[starts-with(@rule,'Unused')]) &gt; 0) or (count(violations/violation[starts-with(@rule,'Empty')]) &gt; 0)">
		<tr>
		  <td>
		  <xsl:choose>
		  	<xsl:when test="../../subsystem">
		  		<xsl:value-of select="../../subsystem/@name"/>
		  	</xsl:when>	
		  	<xsl:otherwise>
		  		Not Classified
		  	</xsl:otherwise>
		  </xsl:choose>
		  </td>
		  <td>                 
		      <a>	
		      <xsl:attribute name="href">classes/<xsl:value-of select="../../@name"/>-classes.html#CL<xsl:value-of select="normalize-space(node())"/></xsl:attribute>		      
		      <xsl:call-template name="split-string-with-string">
					<xsl:with-param name="string" select="node()"/>
					<xsl:with-param name="substring-length" select="100"/>
					<xsl:with-param name="split-string" select="'&#160;'"/>
				</xsl:call-template>					      
		      </a>           
		  </td>
		  <td>          	
		  		<xsl:if test="not(business-critical)">X</xsl:if>          			  	
		  </td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'UnusedPrivateField'])"/></td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'UnusedPrivateMethod'])"/></td>	  
		  <td><xsl:value-of select="count(violations/violation[@rule = 'UnusedLocalVariable'])"/></td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'UnusedFormalParameter'])"/></td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'UnusedImports'])"/></td>
		  
		  <td><xsl:value-of select="count(violations/violation[@rule = 'EmptyCatchBlock'])"/></td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'EmptyIfStmt'])"/></td>	  
		  <td><xsl:value-of select="count(violations/violation[@rule = 'EmptyWhileStmt'])"/></td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'EmptyTryBlock'])"/></td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'EmptyFinallyBlock'])"/></td>
		  <td><xsl:value-of select="count(violations/violation[@rule = 'EmptySwitchStatements'])"/></td>		 
		  
		</tr>
	</xsl:if> 
</xsl:template>  
	 
   
   
</xsl:stylesheet>
