<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xalan="http://xml.apache.org/xalan"
	xmlns:lxslt="http://xml.apache.org/xslt"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
    extension-element-prefixes="redirect">


<xsl:template match="IntegratedAnalysis" mode="scorecard-test">

	<xsl:variable name="packages-element" select="Packages"/>

   	<html>
      <head>
         <link rel="stylesheet" type="text/css" href="stylesheet.css"/>
      </head>
      <body>

         <xsl:call-template name="page-header">
  	  	   <xsl:with-param name="main-menu-page-name" select="'scorecard'"/>
  	  	   <xsl:with-param name="page-name" select="'test'"/>
           <xsl:with-param name="dynamics-page-name" select="'test'"/>
  	  	   <xsl:with-param name="root-element" select="current()"/>
  	     </xsl:call-template>

		<hr size="1"/>
		<br/>
		<h3>Test Metrics</h3>
		For each histogram column, you can roll over the diagrams to get details on the contents. Click on it to go to
		a detailed view. On the bottom of the screen the data is presented in tabular form.
		<br/>
		<table width="100%" class="details">
			<tr>
				<td style="background-color: white;">
			 		<div align="top">
		              <embed width="400" height="300" src="img/subsystem-tests-per-ncss-histogram.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
	            </td>
				<td style="background-color: white;">
			 		<div align="top">
		              <embed width="400" height="300" src="img/subsystem-test-coverage-histogram.svg" name="svgmap" pluginspage="http://www.adobe.com/svg/viewer/install/" type="image/svg+xml"></embed>
		            </div>
				</td>
	        </tr>
	    </table>

		<table width="100%" class="details">
		<xsl:if test="count(source[node()='JUnit']) &gt; 0">
			<tr>
		    	<td style="background-color: white;"><br/><br/></td>
		    </tr>
	    	<xsl:call-template name="metric-header-test"/>
		    <xsl:call-template name="metric-rows-test">
				<xsl:with-param name="packages-element" select="$packages-element"/>
			</xsl:call-template>
		</xsl:if>
 	</table>

      </body>
   </html>

</xsl:template>

<xsl:template name="metric-header-test">

	<tr>
       <th>Test Metrics</th>
		 <th>Test suites</th>
		 <th>Tests</th>
		 <th>Tests per Source Statements</th>
		 <th>Errors</th>
		 <th>Time</th>
		 <th>Line Coverage</th>
		 <th>Line Coverage per Test</th>
    </tr>
</xsl:template>

<xsl:template name="metric-rows-test">
	<xsl:param name="packages-element"/>
	<xsl:param name="row-type"/>

	<!-- Print first the totals -->
	<xsl:variable name="source-statements" select="sum($packages-element/Package/Stats/ncss)"/>
	<xsl:call-template name="subsystem-test-detail">
		<xsl:with-param name="subsystem-name" select="'Total'"/>
		<xsl:with-param name="subsystem-link" select="''"/>

		<xsl:with-param name="suites" select="sum($packages-element/Package/Stats/test-metrics/testsuites)"/>
		<xsl:with-param name="tests" select="sum($packages-element/Package/Stats/test-metrics/tests)"/>
		<xsl:with-param name="errors" select="sum($packages-element/Package/Stats/test-metrics/errors)"/>
		<xsl:with-param name="time" select="sum($packages-element/Package/Stats/test-metrics/time)"/>
		<xsl:with-param name="line-coverage" select="sum($packages-element/Package/Stats/test-metrics[not(source-statements-covered ='NaN') and not(source-statements-covered ='')]/source-statements-covered)
														div $source-statements"/>
		<xsl:with-param name="source-statements" select="$source-statements"/>
	</xsl:call-template>

	<!-- Go through all the subsystems -->
	<xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
		<xsl:variable name="subsystem-name" select="normalize-space(name)"/>
				<xsl:variable name="file-name"><xsl:call-template name="subsystem-url">
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                    <xsl:with-param name="anchor" select="'test-metrics'"/>
                </xsl:call-template></xsl:variable>
				<xsl:variable name="source-statements" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/ncss)"/>
				<xsl:call-template name="subsystem-test-detail">
					<xsl:with-param name="subsystem-name" select="$subsystem-name"/>
					<xsl:with-param name="subsystem-link" select="$file-name"/>

					<xsl:with-param name="suites" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/test-metrics/testsuites)"/>
					<xsl:with-param name="tests" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/test-metrics/tests)"/>
					<xsl:with-param name="errors" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/test-metrics/errors)"/>
					<xsl:with-param name="time" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/test-metrics/time)"/>
					<xsl:with-param name="line-coverage" select="sum($packages-element/Package[subsystem/@name = $subsystem-name]/Stats/test-metrics[not(source-statements-covered ='NaN') and not(source-statements-covered ='')]/source-statements-covered)
																	div $source-statements"/>
					<xsl:with-param name="source-statements" select="$source-statements"/>
				</xsl:call-template>
	</xsl:for-each>
</xsl:template>

<xsl:template name="subsystem-test-detail">
	<xsl:param name="subsystem-name"/>
	<xsl:param name="subsystem-link"/>
	<xsl:param name="suites"/>
	<xsl:param name="tests"/>
	<xsl:param name="errors"/>
	<xsl:param name="time"/>
	<xsl:param name="line-coverage"/>
	<xsl:param name="source-statements"/>

	<tr>
		<td align="left">
		    <xsl:choose>
				<xsl:when test="$subsystem-link != ''">
					<a href="{$subsystem-link}"><xsl:value-of select="$subsystem-name"/></a>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$subsystem-name"/>
				</xsl:otherwise>
			</xsl:choose>
		</td>
		<td><xsl:value-of select="$suites"/></td>
		<td><xsl:value-of select="$tests"/></td>
		<td><xsl:value-of select="format-number($tests div $source-statements, '#.##')"/></td>
		<td><xsl:value-of select="$errors"/></td>
		<td><xsl:value-of select="format-number($time, '#.##')"/></td>
		<td><xsl:value-of select="format-number(100 * $line-coverage, '#.##')"/> %</td>
		<xsl:choose>
			<xsl:when test="($tests &gt; 0)">
				<td><xsl:value-of select="format-number(100 * $line-coverage div $tests, '#.##')"/> %</td>
			</xsl:when>
			<xsl:otherwise>
				<td>0 %</td>
			</xsl:otherwise>
		</xsl:choose>
	</tr>
</xsl:template>

</xsl:stylesheet>
